import os
import re
import requests
import hashlib
import time
import threading
from concurrent.futures import ThreadPoolExecutor, as_completed
from tqdm.auto import tqdm
from urllib.parse import urlparse

# 全局配置
MAX_WORKERS = 80  # 增加线程池数量
TIMEOUT = 60      # 增加超时时间
HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
    'Referer': 'https://www.example.com/'
}

io_lock = threading.Lock()
file_lock = threading.Lock()  # 添加一个线程锁用于文件操作

def parse_episode_data(content):
    """精准解析剧集和URL"""
    url_pattern = re.compile(
        r'\b(https?://[^\s<>"\'()]+)',
        re.IGNORECASE
    )
  
    episodes = []
    anime_name = None
    current_ep = None
  
    for line in content.split('\n'):
        line = line.strip()
      
        # 检测新的番剧开始
        if line.startswith('番剧:'):
            anime_name = line.replace('番剧:', '').strip()
            current_ep = None  # 重置剧集编号
            continue
      
        if not anime_name:
            continue  # 跳过不属于任何番剧的信息
      
        # 检测剧集编号
        ep_match = re.search(r'第\s*(\d+)\s*集', line)
        if ep_match:
            current_ep = ep_match.group(1)
        elif line.startswith('第') and any(char.isdigit() for char in line):  # 增强检测逻辑
            # 尝试从更复杂的格式中提取剧集编号
            digits = re.findall(r'\d+', line)
            if digits:
                current_ep = digits[0]
      
        # 提取URL
        url_match = url_pattern.search(line)
        if url_match and current_ep is not None:  # 确保current_ep不是None
            raw_url = url_match.group(1)
            clean_url = raw_url.rstrip(',;')
            
            # 检查链接是否包含 
            if 'girigirilove.com' in clean_url:
                # 修改链接为  的代理链接
                clean_url = re.sub(r'(https://)([^/]*\.l\.com)(/[^<]+)', r'https://\2\3', clean_url)
            
            episodes.append((anime_name, current_ep, clean_url))
        elif url_match and current_ep is None:  # 处理无法提取剧集编号的情况
            print(f"⚠️ 无法提取剧集编号: {line}")
            # 可以选择跳过或使用默认值
            current_ep = "unknown"
            raw_url = url_match.group(1)
            clean_url = raw_url.rstrip(',;')
            
            # 检查链接是否包含
            if 'girigirilove.com' in clean_url:
                # 修改链接为  的代理链接
                clean_url = re.sub(r'(https://)([^/]*\.l\.com)(/[^<]+)', r'https:///\2\3', clean_url)
            
            episodes.append((anime_name, current_ep, clean_url))
  
    return sorted(
        [(anime, ep, url) for anime, ep, url in episodes if ep is not None],
        key=lambda x: (x[0], int(x[1]) if x[1].isdigit() else 0)
    )

def is_media_url(url):
    """验证媒体链接有效性"""
    try:
        # 增加GET请求作为备选方案
        try:
            with requests.head(url, headers=HEADERS, timeout=5, allow_redirects=True) as res:
                if res.status_code == 200:
                    content_type = res.headers.get('Content-Type', '')
                    return 'video' in content_type or 'audio' in content_type or url.endswith(('.m3u8', '.ts'))
        except:
            # 如果HEAD请求失败，尝试GET请求
            with requests.get(url, headers=HEADERS, stream=True, timeout=5) as res:
                res.raise_for_status()
                content_type = res.headers.get('Content-Type', '')
                return 'video' in content_type or 'audio' in content_type or url.endswith(('.m3u8', '.ts'))
        return False
    except:
        return False

def download_ts_segment(url, save_path, progress_bar):
    """下载单个TS分片"""
    max_retries = 3  # 设置最大重试次数
    chunk_size = 1024 * 1024  # 每次写入1MB
    
    for retry in range(max_retries):
        try:
            with requests.get(url, headers=HEADERS, stream=True, timeout=TIMEOUT) as res:
                res.raise_for_status()
                total_size = int(res.headers.get('content-length', 0))
                progress_bar.total += total_size  # 更新进度条总大小
                
                os.makedirs(os.path.dirname(save_path), exist_ok=True)
                
                with open(save_path, 'wb') as f:
                    for chunk in res.iter_content(chunk_size=chunk_size):
                        if chunk:
                            f.write(chunk)
                            f.flush()  # 立即写入磁盘
                            progress_bar.update(len(chunk))  # 更新进度条
                return True
        except requests.exceptions.RequestException as e:
            print(f"TS分片下载失败（尝试 {retry + 1}/{max_retries}）: {url} ({str(e)})")
            time.sleep(1)  # 短暂等待后重试
        except Exception as e:
            print(f"TS分片下载异常（尝试 {retry + 1}/{max_retries}）: {url} ({str(e)})")
            time.sleep(1)  # 短暂等待后重试
    
    return False

def parse_m3u8(url, base_url=None):
    """解析m3u8文件，返回ts分片列表"""
    try:
        response = requests.get(url, headers=HEADERS, timeout=TIMEOUT)
        response.raise_for_status()
        content = response.text
        lines = content.splitlines()
        ts_urls = []
        for line in lines:
            line = line.strip()
            if line and not line.startswith('#'):
                if base_url and not line.startswith('http'):
                    ts_url = f"{base_url}/{line}"
                else:
                    ts_url = line
                ts_urls.append(ts_url)
        return ts_urls
    except Exception as e:
        print(f"解析m3u8失败: {str(e)}")
        return []

def download_m3u8(url, save_dir, ep_num, anime_name, log_path):
    """下载m3u8指向的ts分片并合并"""
    parsed_url = urlparse(url)
    base_url = f"{parsed_url.scheme}://{parsed_url.netloc}{os.path.dirname(parsed_url.path)}"
    
    ts_urls = parse_m3u8(url, base_url)
    if not ts_urls:
        return False, f"第{ep_num}集 (无法解析m3u8)"
    
    temp_dir = os.path.join(save_dir, f"temp_{ep_num}")
    os.makedirs(temp_dir, exist_ok=True)
    
    # 创建一个进度条
    progress_bar = tqdm(total=0, unit='B', unit_scale=True, desc=f"第{ep_num}集", leave=False)
    
    futures = []
    with ThreadPoolExecutor(max_workers=MAX_WORKERS) as executor:
        for i, ts_url in enumerate(ts_urls):
            ts_save_path = os.path.join(temp_dir, f"segment_{i:04d}.ts")
            futures.append(executor.submit(download_ts_segment, ts_url, ts_save_path, progress_bar))
        
        # 收集所有下载结果
        download_results = [future.result() for future in as_completed(futures)]
        
        # 检查是否有下载失败的分片
        if not all(download_results):
            import shutil
            shutil.rmtree(temp_dir, ignore_errors=True)
            progress_bar.close()
            return False, f"第{ep_num}集 (部分ts分片下载失败)"
    
    progress_bar.close()
    
    # 合并ts文件
    merged_path = os.path.join(save_dir, f"第{ep_num}集.mp4")
    try:
        with open(merged_path, 'wb') as merged_file:
            for i in range(len(ts_urls)):
                ts_path = os.path.join(temp_dir, f"segment_{i:04d}.ts")
                if os.path.exists(ts_path):
                    with open(ts_path, 'rb') as ts_file:
                        merged_file.write(ts_file.read())
                else:
                    raise FileNotFoundError(f"分片 {i} 不存在")
        # 清理临时文件
        import shutil
        shutil.rmtree(temp_dir)
        
        # 检查文件大小，如果过小则视为无效文件
        if os.path.exists(merged_path):
            file_size = os.path.getsize(merged_path)
            if file_size < 10 * 1024 * 1024:  # 小于10MB视为无效
                os.remove(merged_path)
                return False, f"第{ep_num}集 (文件大小异常)"
        
        return True, merged_path
    except Exception as e:
        print(f"合并ts文件失败: {str(e)}")
        return False, f"第{ep_num}集 (合并ts分片失败)"

def generate_episode_links(base_url, max_episodes=24):
    """从基础链接生成连续集数的下载链接，并检查链接有效性"""
    episode_links = []
    parsed_url = urlparse(base_url)
    path_parts = parsed_url.path.split('/')
    
    # 找到集数所在的位置（倒数第二个部分）
    if len(path_parts) < 2:
        print(f"无法解析基础链接结构: {base_url}")
        return episode_links
    
    episode_index = -2  # 集数在路径中的位置（倒数第二个部分）
    
    for ep_num in range(1, max_episodes + 1):
        # 格式化集数为两位数字（例如：01, 02, ..., 24）
        formatted_ep = f"{ep_num:02d}"
        
        # 替换基础链接中的集数部分
        new_path_parts = path_parts.copy()
        if len(new_path_parts) > abs(episode_index):
            new_path_parts[episode_index] = formatted_ep
        else:
            print(f"无法找到集数部分: {base_url}")
            break
        
        new_path = '/'.join(new_path_parts)
        new_url = f"{parsed_url.scheme}://{parsed_url.netloc}{new_path}"
        
        # 检查链接是否有效
        if is_media_url(new_url):
            episode_links.append((formatted_ep, new_url))
        else:
            print(f"链接无效: {new_url}")
            # 如果连续多个链接无效，可以提前停止生成
            if ep_num > 1 and len(episode_links) == 0:
                print(f"停止生成更多链接: {new_url}")
                break
    
    return episode_links

def process_anime_list(txt_path, root_dir):
    """处理动漫列表"""
    if not os.path.exists(txt_path):
        print(f"视频列表文件不存在: {txt_path}")
        return
      
    print(f"\n{'='*40}")
    print(f"处理视频列表: {os.path.basename(txt_path)}")
    print(f"{'='*40}")
      
    try:
        # 读取下载记录
        anime_dirs = {}
        with io_lock:
            for entry in os.scandir(root_dir):
                if entry.is_dir():
                    log_path = os.path.join(entry.path, ".download_log")
                    downloaded = set()
                    if os.path.exists(log_path):
                        with open(log_path, 'r') as f:
                            downloaded = set(f.read().splitlines())
                    anime_dirs[entry.name] = downloaded

        # 解析视频列表
        with open(txt_path, 'r', encoding='utf-8', errors='ignore') as f:
            content = f.read()
          
        episodes = parse_episode_data(content)
        if not episodes:
            print("未找到有效下载链接")
            return
              
        # 按番剧分组
        anime_groups = {}
        for anime_name, ep_num, url in episodes:
            if anime_name not in anime_groups:
                anime_groups[anime_name] = []
            anime_groups[anime_name].append((ep_num, url))

        total_animes = len(anime_groups)
        completed_animes = 0
        failed_animes = []

        # 依次处理每个番剧
        for anime_name in anime_groups:
            print(f"\n开始下载 ({completed_animes + 1}/{total_animes}): {anime_name}")
            failed_episodes = []
            consecutive_failures = 0  # 连续失败计数
            
            # 获取基础链接（假设第一个链接是基础链接）
            base_url = None
            for ep_num, url in anime_groups[anime_name]:
                if url.endswith('.m3u8'):
                    base_url = url
                    break
            
            if not base_url:
                print(f"未找到m3u8基础链接: {anime_name}")
                continue
            
            # 生成1-24集的下载链接
            episode_links = generate_episode_links(base_url)
            
            for ep_num, url in episode_links:
                anime_dir = os.path.join(root_dir, anime_name)
                os.makedirs(anime_dir, exist_ok=True)
                log_path = os.path.join(anime_dir, ".download_log")
                
                # 检查文件是否已经存在
                merged_path = os.path.join(anime_dir, f"第{ep_num}集.mp4")
                if os.path.exists(merged_path):
                    # 检查文件大小，如果过小则视为无效文件
                    file_size = os.path.getsize(merged_path)
                    if file_size < 10 * 1024 * 1024:  # 小于10MB视为无效
                        os.remove(merged_path)
                        print(f"文件大小异常，重新下载: 第{ep_num}集.mp4 (番剧: {anime_name})")
                    else:
                        print(f"已下载: 第{ep_num}集.mp4 (番剧: {anime_name})")
                        consecutive_failures = 0  # 重置连续失败计数
                        continue
                # 检查下载记录
                if anime_name in anime_dirs and f"第{ep_num}集.mp4" in anime_dirs.get(anime_name, []):
                    print(f"已下载: 第{ep_num}集.mp4 (番剧: {anime_name})")
                    consecutive_failures = 0  # 重置连续失败计数
                    continue
                
                # 检查链接是否包含 
                if 'girigirilove.com' in url:
                    # 修改链接为  的代理链接
                    url = re.sub(r'(https://)([^/]*\.l\.com)(/[^<]+)', r'https://\2\3', url)
                
                # 下载m3u8指向的ts分片并合并
                result = download_m3u8(url, anime_dir, ep_num, anime_name, log_path)
                success, msg = result
                if not success:
                    failed_episodes.append(msg)
                    consecutive_failures += 1
                    # 如果连续失败次数超过2次，停止下载当前番剧
                    if consecutive_failures > 1:
                        print(f"连续失败次数超过2次，停止下载当前番剧: {anime_name}")
                        break
                else:
                    consecutive_failures = 0  # 重置连续失败计数
                    with io_lock:
                        with open(log_path, 'a') as log_f:
                            log_f.write(f"第{ep_num}集.mp4\n")
            
            completed_animes += 1

            # 检查是否有下载失败的剧集
            if failed_episodes:
                failed_animes.append((anime_name, failed_episodes))
                print(f"\n番剧下载部分失败 ({completed_animes}/{total_animes}): {anime_name}")
                print("下载失败的剧集:")
                for ep in failed_episodes:
                    print(f"❌ {ep}")
            else:
                print(f"\n番剧下载完成 ({completed_animes}/{total_animes}): {anime_name}")
          
        print(f"\n全部下载完成: {txt_path}")

        # 报告下载失败的番剧和剧集
        if failed_animes:
            print("\n下载失败的番剧和剧集:")
            for anime_name, failed_episodes in failed_animes:
                print(f"番剧: {anime_name}")
                for ep in failed_episodes:
                    print(f"  - {ep}")
            print("建议检查网络连接、链接有效性，或稍后重试。")
      
    except Exception as e:
        print(f"处理异常: {str(e)}")

if __name__ == "__main__":
    ANIME_ROOT = "H:\q\动漫"
    TXT_PATH = os.path.join(ANIME_ROOT, "视频.txt")
    print("动漫下载器启动")
    process_anime_list(TXT_PATH, ANIME_ROOT)
    print("全部任务完成")